/*
 *  Copyright (c) 2023 Fraunhofer Institute for Applied Information Technology
 *
 *  This program and the accompanying materials are made available under the
 *  terms of the Apache License, Version 2.0 which is available at
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

package org.fraunhofer.fit.extension.extractor;

import org.eclipse.edc.spi.monitor.Monitor;
import org.fraunhofer.fit.extension.extractor.model.FairvalidatorModel;
import org.fraunhofer.fit.extension.extractor.model.ValidationResponseModel;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Map;


public class MetadataExtractor {

    private final Monitor monitor;

    public MetadataExtractor(Monitor monitor) {
        this.monitor = monitor;
    }

    public  Map<String, Object> getFairScore(String filePath, String pid, String dataLink, String version) {

        monitor.info("================ Running Fair Score evaluator ========");

        FairvalidatorModel faiRvalidatorModel = extractBasicMetadata(filePath, pid, dataLink, version);

        if (faiRvalidatorModel.getMimeType().startsWith("application/")) {
            TextMetadataExtractor textMetadataExtractor = new TextMetadataExtractor(filePath, monitor);
            faiRvalidatorModel = textMetadataExtractor.getTextMetadata(faiRvalidatorModel);
        }

        FairService fairService = new FairService(monitor);
        ValidationResponseModel validationResponseModel = fairService.getValidation(faiRvalidatorModel);

        faiRvalidatorModel.setValidationScore(validationResponseModel.getValidationScore());
        faiRvalidatorModel.setValidationScoreDesc(validationResponseModel.getValidationScoreDesc());

        faiRvalidatorModel.print();
        validationResponseModel.print();

        monitor.info("================ Fair Score evaluator Completed ========");
        return faiRvalidatorModel.getProperties();
    }

    private FairvalidatorModel extractBasicMetadata(String filePath, String pid, String dataLink, String version) {

        FairvalidatorModel metadata = new FairvalidatorModel();
        File file = new File(filePath);

        if (file.isFile() && file.exists()) {
            Path path = file.toPath();
            try {
                Path absolutePath = Paths.get(file.getAbsolutePath());
                BasicFileAttributes basicAttributes = Files.readAttributes(absolutePath, BasicFileAttributes.class);
                // These metadatas are extracted from the file properties.
                metadata.setName(file.getName());
                metadata.setByteSize((int) file.length());
                metadata.setPublicationDate(basicAttributes.creationTime().toString());
                metadata.setModifiedDate(basicAttributes.lastModifiedTime().toString());
                metadata.setMimeType(Files.probeContentType(path));
                metadata.setPublisher(String.valueOf(Files.getOwner(path)));
                // These metadata are provided by user.
                metadata.setPid(String.valueOf(pid));
                metadata.setLicence("");
                metadata.setDataLink(dataLink);
                metadata.setContactPoint("");
                metadata.setVersion(version);
                metadata.setDataAccessLevel(""); // access level will be infer by PID and DataLink

            } catch (IOException e) {
                e.printStackTrace();
            }

        } else {
            monitor.info(filePath + " either does exist or its not a file");
        }
        return metadata;
    }
}