/*
 *  Copyright (c) 2023 Fraunhofer Institute for Applied Information Technology
 *
 *  This program and the accompanying materials are made available under the
 *  terms of the Apache License, Version 2.0 which is available at
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

package org.fraunhofer.fit.extension.extractor;

import org.eclipse.edc.spi.monitor.Monitor;
import org.fraunhofer.fit.extension.extractor.model.FairvalidatorModel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class TextMetadataExtractor {

    private String filepath;
    private String summary;
    private String language;
    private String keywords;
    private String title;
    private Monitor monitor;

    public TextMetadataExtractor(String filePath, Monitor monitor) {
        this.filepath = filePath;
        this.monitor = monitor;
        monitor.info("=== Getting TextMetadata for file ====" + this.filepath);
        loadPythonScript();
    }

    private void loadPythonScript() {
        try {
            monitor.info("Running python interpreter");

            InputStream inputStream = TextMetadataExtractor.class.getResourceAsStream("/text_metadata_extractor.py");
            byte[] scriiptBytes = inputStream.readAllBytes();

            File tempFile = File.createTempFile("script", ".py");
            FileOutputStream outputStream = new FileOutputStream(tempFile);
            outputStream.write(scriiptBytes);

            List<String> command = new ArrayList<>();
            command.add("python");
            command.add(tempFile.getAbsolutePath());
            command.add(this.filepath);


            // Create ProcessBuilder instance
            ProcessBuilder processBuilder = new ProcessBuilder(command);
            processBuilder.redirectErrorStream(true); // Redirect stderr to stdout

            // Start the process
            Process process = processBuilder.start();

            // Read the output of the script
            BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            List<String> lines = new ArrayList<>();
            String  line;
            while ((line = reader.readLine()) != null) {
                if (line.startsWith("python info")) {
                    lines.add(line.split("splithere")[1]);
                    System.out.println(line);
                } else {
                    monitor.info(line);
                }

            }

            this.language = lines.get(0);
            this.summary = lines.get(1);
            this.keywords = lines.get(2);
            this.title = lines.get(3);

            // Wait for the process to complete
            int exitCode = process.waitFor();

            // Print the exit code
            System.out.println("Exit code: " + exitCode);
            tempFile.delete();

        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        }

    }

    public FairvalidatorModel getTextMetadata(FairvalidatorModel faiRvalidatorModel) {
        faiRvalidatorModel.setTitle(this.title);
        faiRvalidatorModel.setDescription(this.summary);
        faiRvalidatorModel.setKeywords(this.keywords.split(","));
        faiRvalidatorModel.setLanguage(this.language);

        return faiRvalidatorModel;
    }

}
